import { controller } from "@/components/treeview/signals";
import { globalLookup } from "./global";
import { WaveContainerView } from "./wave-container-view";

export const recoverConfig = {
    /**
     * @type {Map<string, IRenderOption>}
     */
    waves: new Map(),
};

async function findRecoverFile(recoverJsonPath) {
    try {
        const response = await fetch(recoverJsonPath);
        if (!response.ok) {
            return undefined;
        }
        const recoverJson = await response.json();
        return recoverJson;
    } catch (error) {
        return undefined;
    }
}

/**
 * 
 * @param {string} inputFile 读取的 vcd 文件的路径
 */
export async function recoverFromInputFile(inputFile) {
    const recoverJsonPath = inputFile + '.pkl';
    const recoverJson = await findRecoverFile(recoverJsonPath);
    if (recoverJson) {
        const waves = recoverJson.waves;
        if (waves instanceof Array && waves.length > 0) {
            for (const wave of waves) {
                const name = wave.name;
                const option = wave.option;
                if (name === undefined || option === undefined || recoverConfig.waves.has(name)) {
                    continue;
                }
                recoverConfig.waves.set(name, option);
            }
        }
    }
}

/**
 * @description 恢复现场的函数，主要恢复两个变量 currentWiresRenderView 和 currentSignalRenderOptions
 * @param {TopModWireItem[]} topModules
 */
export function recoverSession(topModules) {
    // 匹配前缀树
    

    controller.lastSignal = undefined;
    globalLookup.render();
}