/**
 * @description
 * @typedef {Object} WaveElements
 * @property {HTMLDivElement} grid
 * @property {HTMLDivElement} view
 * @property {HTMLDivElement} values
 */

/**
 * @description
 * @typedef {Object} ChangoItem
 * @property {string} kind
 * @property {Array<number | string>} wave
 * @property {WebGLVertexArrayObject} lineVao
 * @property {WebGLVertexArrayObject} maskVao
 */

/**
 * @description 装载在 globalLookup 中的记录 vcd 数据的字典
 * @typedef {Record<string, ChangoItem>} Chango
 */

/**
 * @description
 * @typedef {Object} WireItemBaseInfo
 * @property {string} link
 * @property {string} name
 * @property {number} size
 * @property {string | undefined} parentLink 如果当前的波形被加入了一个 group 或 bus 中，该值则不会为 undefined
 */


/**
 * @description
 * @typedef {Object} WaveRenderSidebarItem
 * @property {WireItemBaseInfo} signalInfo
 * @property {GroupBaseInfo} groupInfo
 * @property {0 | 1 | 2} renderType 0：单条波形，1：分组，2：bus聚合
 * @property {WaveRenderSidebarItem[]} children 分组内的波形，只有当 renderType 为 1 时才不为空数组
 */

/**
 * @description 
 * @typedef {Object} WireItem
 * @property {string} kind
 * @property {string} link
 * @property {string} name
 * @property {number} size
 * @property {WireItem} parent
 * @property {string} type
 */

/**
 * @description
 * @typedef {Object} GroupBaseInfo
 * @property {string} name
 * @property {string} color
 * @property {boolean} collapse
 */


/**
 * @description 
 * @typedef {Object} TopModWireItem
 * @property {string} kind
 * @property {string} link
 * @property {string} name
 * @property {number} size
 * @property {string} type
 * @property {(TopModWireItem | WireItem)[]} body
 */

/**
 * @description
 * @typedef {Array<{ ref: string }>} View
 */

/**
 * @description
 * @typedef {Set<WireItem>} CurrentWires
 */

/**
 * @description
 * @typedef {Object} GlobalLookup
 * @property {Chango} chango
 * @property {View} view
 * @property {CurrentWires} currentWires
 * @property {Map<string, WireItem>} link2CurrentWires
 * @property {WaveRenderSidebarItem[]} currentWiresRenderView
 * @property {Record<string, BigInt | string>} currentSignalValues
 * @property {Set<string>} sidebarSelectedWireLinks
 * @property {Map<string, IRenderOption>} currentSignalRenderOptions
 * @property {number} time
 */

/**
 * @description
 * @typedef {Object} Pstate
 * @property {number} tgcd
 * @property {number} width
 * @property {number} height
 * @property {number} xScale
 * @property {number} xCursor
 * @property {number} xOffset
 * @property {number} yOffset
 * @property {number} yStep
 * @property {number} yDuty
 * @property {number} sidebarWidth
 * @property {number} topBarHeight
 * @property {number} botBarHeight
 */

/**
 * @description
 * @typedef {Object} Rgba
 * @property {number} red
 * @property {number} green
 * @property {number} blue
 * @property {number} alpha
 */

/**
 * @description
 * @typedef {Object} Updater
 * @property {number} index
 * @property {Rgba} rgba
 */

/**
 * @description
 * @typedef {Object} RenderConfig
 * @property {'common' | 'action' | 'value' | 'wave-only' | undefined} type
 */

/**
 * @typedef {Object} Handler
 * @property {string} desc
 * @property {(arg1: Pstate, arg2: any) => boolean} fn
 */

/**
 * @typedef {Object} Deso
 * @property {(config: RenderConfig) => void} render
 * 
 */

/**
 * @typedef {'scrollLeft' | 'scrollRight' | 'scrollUp' | 'scrollDown' | 'scaleUp' | 'scaleDown'} EventHandlerKind
 */


/**
 * @typedef {Object} IRenderOption
 * @property {number | undefined} height 波形的高度，默认为 30，可以在右侧的栏目中设置
 * @property {number | undefined} color  波形的颜色，可以在右击菜单栏中设置
 * @property {number | undefined} valueFormat 波形数字的展示形式，可以在上侧的下拉菜单中设置，vec 类型的波形，可以设置为：0：二进制、1：八进制、2：十六进制、3：有符号整型、4：无符号整型、5：浮点数（半精度、6：单精度、7：双精度）
 * @property {number | undefined} renderModal 波形的渲染模式，默认为 bit 和 vec，vec 类型的波形，可以设置为：0：数字形式、1：折线模拟形式、2：阶梯模拟形式
 * @property {number | undefined} highlight 当前是否需要进行高亮 1 代表需要
 */

